package RHU;

use strict;

# -----------------------------------------------------------------------------

my $date_time = `date`;

$date_time =~ s/\s+/\_/g;

# -----------------------------------------------------------------------------
# --

sub new() {

  my $class = shift;
  my $args  = shift;
  
    my $rootdir  = "$ENV{HOME}/__updates";
    my $self = { };

    bless($self, $class);

    $self->{ROOTDIR} = $rootdir;
    $self->{ARGS}    = $args;
    $self->{TIME}    = time; 

    $self->log("v", 0, "simple", ["Assumed local updates dir: $rootdir"]);

    unless ((-e $rootdir) && (-d $rootdir)) {
        die "\n\n $rootdir must exist and be a directory! \n\n";
      }

    return $self;
  }


# -----------------------------------------------------------------------------
# --

sub rootdir() {
    my $self = shift;

    return $self->{ROOTDIR};  
  }


# -----------------------------------------------------------------------------
# --

sub get_download() {

  my $self    = shift;
  my $default = shift;

    my $answer = $default;
    my $input  = "";

    ###print "\n";

    while (1) {
        ###print "\n ...Add this rpm to download list? [$default] ";
        $self->log("v", 2, "simple", 
                   ["...Add this rpm to download list? [$default] "]);

        $input = <STDIN>;
        chomp($input);

        if ($input =~ m/\S/) {
            if (($input eq "y") || ($input eq "Y")) {
                $answer = "y";
                last;
              }
            elsif (($input eq "n") || ($input eq "N")) {
                $answer = "n";
                last;
              }
          }
        else {
            last;
          }
      }
    return $answer;
  }


# -----------------------------------------------------------------------------
# -- returns true only if a-vers-rels is newer than b-vers-rels :

sub newer_versrel() {
    my $self = shift;
    my ($a_vers, $a_rels, $b_vers, $b_rels) = @_;

    # -- version appears to be of the form 
    #            
    #        <num>.<num>, 
    #        <num>.<num><lett>, 
    #        <num>.<num>.<num> or
    #        <num>.<num>.<num><lett>, 
    #            

    my @a_vers = split(/\./, $a_vers);
    my @b_vers = split(/\./, $b_vers);

    my @a_rels = split(/\./, $a_rels);
    my @b_rels = split(/\./, $b_rels);

##print "\n Is this: $a_vers-$a_rels, newer than this: $b_vers-$b_rels ?";

##  if (   ($a_vers gt $b_vers)
    if (   &vers_gt(\@a_vers, \@b_vers)
        || (($a_vers eq $b_vers) && &generalised_gt(\@a_rels, \@b_rels, 0))) {
        return 1;
      }
    else {
        return 0;
      }
  }


sub vers_gt() {
    my $a = shift;
    my $b = shift;

    my $i = 0;

    foreach (@$a) {
        unless (defined $b->[$i]) {return 1}
        if ($a->[$i] < $b->[$i])  {return 0}
        if ($a->[$i] > $b->[$i])  {return 1}
        $i++;
      }
###    return 0;
  }


# -- is $a greater than (newer than) $b ?

sub generalised_gt() {
    my $a       = shift;
    my $b       = shift;
    my $start_i = shift;

    my $i;

    for ($i = $start_i; $i <= @$a - 1; $i++) {
    
        if ((defined $a->[$i]) && (defined $b->[$i])) {
            unless (    ($a->[$i] =~ m/^\d*[a-z]*$/) 
                     && ($b->[$i] =~ m/^\d*[a-z]*$/)){
                die "\n\n Unknown format: $a->[$i], $b->[$i] \n\n";
              }        
            my $a_n = $a->[$i];
            my $a_l = $a->[$i];
            my $b_n = $b->[$i];
            my $b_l = $b->[$i];

            $a_n =~ s/[a-z]*$//;
            $b_n =~ s/[a-z]*$//;

            $a_l =~ s/^\d*//;
            $b_l =~ s/^\d*//;

            if ($a_n > $b_n) {
                return 1;
              }
            elsif (($a_n == $b_n) && ($a_l gt $b_l)) {
                return 1;
              }
            else {
                return &generalised_gt($a, $b, $start_i + 1);
              }
          }
        elsif (defined $a->[$i]) {
            return 1;
          }
#       elsif (defined $b[$i]) {
#           return 0;
#         }
        else {
            return 0;
          }

      }
  }


# -----------------------------------------------------------------------------
# --

sub log() {
    my $self         = shift;
    my $verbosity    = shift;
    my $indent_level = shift;
    my $style        = shift;
    my $string       = shift;

    my ($i, $j);

    # -- unless suitable verbosity level, exit :

    if ($self->{ARGS}->vvv()) {
        unless (($verbosity eq "vvv") || ($verbosity eq "vv") 
                                      || ($verbosity eq "v")  ) {
            return 1;
          }
      }
    elsif ($self->{ARGS}->vv()) {
        unless (($verbosity eq "v") || ($verbosity eq "vv")) {
            return 1;
          }
      }
    elsif ($self->{ARGS}->v()) {
        unless ($verbosity eq "v") {
            return 1;
          }
      }
    else {
        return 1;
      }


    # -- print :

    if (($style eq "simple") || ($style eq "s")) {
        for ($i = 0; $i <= $#$string; $i++) {
            $self->echo_and_log("\n ");
            for ($j = 1; $j <= $indent_level; $j++) {
                $self->echo_and_log("    ");
              }
            $self->echo_and_log($string->[$i]);
          }
      }
    elsif ($style eq "indent_after_one") {
        $self->echo_and_log("\n ");
        for ($i = 1; $i <= $indent_level; $i++) {
            $self->echo_and_log("    ");
          }
        $self->echo_and_log($string->[0]);

        for ($i = 1; $i <= $#$string; $i++) {
            $self->echo_and_log("\n ");
            for ($j = 1; $j <= $indent_level + 1; $j++) {
                $self->echo_and_log("    ");
              }
            $self->echo_and_log($string->[$i]);
          }
      }
    else {
        die "\n\n (caller(0))[3] :  Unknown style: $style \n\n";
      }
  }

# -----------------------------------------------------------------------------
# --

sub echo_and_log() {
    my $self   = shift;
    my $string = shift;

    print $string;

    open(LOG, ">>rhupdate.LOG.$date_time");
    print LOG $string;
    close(LOG);
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
