package rhu_RPMs_Installed;

use strict;
use vars qw(@ISA);

@ISA = ("rhu_RPMs_Template");


# -----------------------------------------------------------------------------
# -- CONSTRUCT :
# -----------------------------------------------------------------------------

# -- return an object representing the locally-installed RPMs (cf. rpm -qa);
#    attributes include RPM names, versions and release-numbers :

sub new() {

  my $class = shift;
  my $args  = shift;
  my $rhu   = shift;

    my $self = { };

    bless($self, $class);

    $self->{CLASS} = $class;
    $self->{RHU}   = $rhu;
    $self->{ARGS}  = $args;

    ($self->{NAMES}, $self->{VERS}, $self->{RELS}) 
                                 = $self->get_list_of_locally_installed_rpms();

    # -- if madly-verbose, list locally-installed RPMs (by name) :
    $self->{RHU}->log("vvv", 1, "simple", 
                              ["Locally-installed RPMs are :"]);
    foreach (sort keys %{$self->{NAMES}}) {
        $self->{RHU}->log("vvv", 2,  "simple", [$self->{NAMES}->{$_}]);
      }  

    return $self;
  }


# -----------------------------------------------------------------------------
# -- CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

# -- run "rpm -qa" and extract info from it --- return name-keyed hashes of 
#    rpm-names, versions and release-numbers :

sub get_list_of_locally_installed_rpms() {

  my $self = shift;

    my $rpm_list = `/bin/rpm -qa`;
    my @rpm_list = split(/\n/, $rpm_list);

    my ($names, $versions,  $releases);

    foreach my $rpm (@rpm_list) {
        my @rpm = split(/\-/, $rpm);

        my $release = $rpm[$#rpm];
        my $version = $rpm[$#rpm - 1];

        my $name = $rpm;
        $name =~ s/\-$version\-$release$//;

        $names->{$name}    = $name;
        $versions->{$name} = $version;
        $releases->{$name} = $release;
      }

    return ($names, $versions, $releases);
  }


# -----------------------------------------------------------------------------
# -- PUBLIC READ METHODS :
# -----------------------------------------------------------------------------

# -- return pointer to (unsorted) list (array) of installed RPM package names :

sub rpm_names() {

  my $self = shift;

    my @keys = keys %{$self->{NAMES}};
    return \@keys;
  }


sub rpm_exists() {
  my $self = shift;
  my $name = shift;

    if (defined $self->{NAMES}->{$name}) {
        return 1;
      }
    else {
        return 0;
      }
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
